function [C_pos,C_neg,Ctot_pos,Ctot_neg,B_pos,B_neg] = clustering_coef_wu_sign(W,coef_type)
%CLUSTERING_COEF_WU_SIGN     Multiple generalizations of the clustering coefficient 
%
%   [C_pos,C_neg,Ctot_pos,Ctot_neg] = clustering_coef_wu_sign(W,coef_type);
%
%   The weighted clustering coefficient is the average weight or intensity
%   of all triangles associated with each node.
%
%   Inputs:
%       W,          
%           Weighted undirected connection matrix
%
%       corr_type,
%           Desired type of clustering coefficient.
%           Options:  
%           1,  (default) Onnela et al. formula, used in original
%               clustering_coef_wu.m. Computed separately for positive &
%               negative weights.
%           2,  Zhang & Horvath formula, similar to Onnela formula except
%               denominator of Onnela formula relies on binarizing the
%               network whereas this denominator is based on weight value,
%               which reduces the sensitivity of this measure to the
%               weights directly connected to the node of interest.
%               Computed separately for positive & negative weights.
%           3,  Constantini & Perugini's generalization of the Zhang &
%               Horvath formula. This formula takes both positive &
%               negative weights into account simultaneously, & is
%               particularly sensitive to non-redundancy in path
%               information based on sign (i.e., when two weights are
%               positive & one negative, or all three are negative, both of
%               which indicate that the weight of the third path is not
%               redundant information). Produces only one value.
%
%
%   Outputs: 
%       C_pos/C_neg,
%           Clustering coefficient vector for positive/negative weights.
%           For the third option, only one vector is outputted (as C_pos). 
%       Ctot_pos/Ctot_neg,
%           Mean clustering coefficient for positive and negative weights.
%
%   References: 
%       Onnela et al. (2005) Phys Rev E 71:065103
%       Zhang & Horvath (2005) Stat Appl Genet Mol Biol 41:1544-6115
%       Costantini & Perugini (2014) PLOS ONE 9:e88669
%
%
%   Contributor: Jeff Spielberg, Boston University, 2014-2015
%                (script based on clustering_coef_wu.m)

%
%   Modification History:
%   May 2014: Added computation of pos & neg weights separately & 
%             computation of mean coefficient (Jeff Spielberg)
%   May 2015: Added computation of Zhang & Horvath and Constantini & 
%             Perugini formulas (Jeff Spielberg)
%   May 2016: Bugfix in computation of the denominator of the Costantini &
%             Perugini (flag 3) version (Chiara Pintossi)

if ~exist('coef_type','var')
    coef_type = 1;
end

n            = size(W,1); % Number of nodes
W(1:n+1:end) = 0;         % Remove diagonal

if coef_type==3
    cyc3 = zeros(n,1);                                    % Preallocate space for 3-cycles
    cyc2 = zeros(n,1);                                    % Preallocate space for denominator
    for i = 1:n                                           % For each node of interest
        for j = 1:n                                       % For each 1st neighbor
            for q = 1:n                                   % For each 2nd neighbor
                cyc3(i) = cyc3(i)+(W(j,i)*W(i,q)*W(j,q)); % Find 3-cycles
                if j~=q                                   % If the neighbors are not the same
                    cyc2(i) = cyc2(i)+abs(W(j,i)*W(i,q)); % Find denominator
                end
            end
        end
    end
    cyc2(cyc3 == 0) = inf;              % If no 3-cycles exist, make C=0 (via K=inf)
    C_pos           = cyc3./cyc2;       % Compute clustering coefficient
    Ctot_pos        = mean(C_pos);      % Compute mean clustering coefficient
    B_pos           = nan;              % Set to NaN
    C_neg           = nan(size(C_pos)); % Set negative to NaN
    Ctot_neg        = nan;              % Set negative to NaN
    B_neg           = nan;              % Set negative to NaN
else
    W_pos    = W.*(W>0);                              % Extract positive weight matrix
    W_neg    = -W.*(W<0);                             % Extract negative weight matrix
    C_pos    = cco_sub(W_pos,coef_type);              % Compute positive clustering coefficient
    Ctot_pos = mean(C_pos);                           % Compute mean positive clustering coefficient
    St_pos   = [ones(n,1),log(sum(W,2))];             % Compute node strength design matrix
    Cl_pos   = log(C_pos);                            % Transform clustering coefficient
    B_pos    = (St_pos'*St_pos)\(St_pos'*Cl_pos);     % Compute betas
    B_pos    = -B_pos(2);                             % Grab beta of interest and reverse sign
    if any(W_neg>0)                                   % If any negative weights are present
        C_neg    = cco_sub(W_neg,coef_type);          % Compute negative clustering coefficient
        Ctot_neg = mean(C_neg);                       % Compute mean negative clustering coefficient
        St_neg   = [ones(n,1),log(sum(W,2))];         % Compute node strength design matrix
        Cl_neg   = log(C_neg);                        % Transform clustering coefficient
        B_neg    = (St_neg'*St_neg)\(St_neg'*Cl_neg); % Compute betas
        B_neg    = -B_neg(2);                         % Grab beta of interest and reverse sign
    else
        C_neg    = nan(size(C_pos)); % Set negative to NaN
        Ctot_neg = nan;              % Set negative to NaN
        B_neg    = nan;              % Set negative to NaN
    end
end

    function C = cco_sub(W_,coef_type)
        n_ = size(W_,1); % Number of nodes
        switch coef_type 
            case 1
                K_           = sum(W_~=0,2);        % Compute node degrees
                cyc3_        = diag((W_.^(1/3))^3); % Find 3-cycles
                K_(cyc3_==0) = inf;                 % If no 3-cycles exist, make C=0
                C            = cyc3_./(K_.*(K_-1)); % Compute clustering coefficient
            case 2
                cyc3_           = diag(W_^3);                  % Find 3-cycles
                cyc2_           = ((sum(W_,2).^2)-diag(W_^2)); % Compute denominator
                cyc2_(cyc3_==0) = inf;                         % If no 3-cycles exist, make C=0
                C               = cyc3_./cyc2_;                % Compute clustering coefficient
            case 4
                A_   = W_>0;        % Create binary connectivity matrix
                num_ = zeros(n_,1); % Preallocate space
                den_ = zeros(n_,1); % Preallocate space
                mw_  = 1/maxn(W_);  % Invert max
                for i_ = 1:n_
                    for j_ = i_+1:n_
                        for q_ = j_+1:n_
                            num_(i_) = num_(i_)+(2/((.5/(1/W_(i_,j_))+(1/W_(i_,q_)))+(1/W_(j_,q_))))*A_(i_,j_)*A_(i_,q_);
                            num_(j_) = num_(j_)+(2/((.5/(1/W_(j_,i_))+(1/W_(j_,q_)))+(1/W_(i_,q_))))*A_(j_,i_)*A_(j_,q_);
                            num_(q_) = num_(q_)+(2/((.5/(1/W_(q_,j_))+(1/W_(q_,i_)))+(1/W_(j_,i_))))*A_(q_,j_)*A_(q_,i_);
                            den_(i_) = den_(i_)+(2/((.5/(1/W_(i_,j_))+(1/W_(i_,q_)))+mw_))*A_(i_,j_)*A_(i_,q_);
                            den_(j_) = den_(j_)+(2/((.5/(1/W_(j_,i_))+(1/W_(j_,q_)))+mw_))*A_(j_,i_)*A_(j_,q_);
                            den_(q_) = den_(q_)+(2/((.5/(1/W_(q_,j_))+(1/W_(q_,i_)))+mw_))*A_(q_,j_)*A_(q_,i_);
                        end
                    end
                end
                den_(num_==0) = inf;        % If no 3-cycles exist, make C=0
                C             = num_./den_; % Compute clustering coefficient                
        end
