function w = comm_bet_cent_wu(W)

%   Communicability Betweenness Centrality
%
%   w = comm_bet_cent_wu(W);
%
%   Computes betweenness centrality based on the communicability framework. 
%   The communicability framework takes into account all possible paths 
%   between nodes, not just the shortest. This may be more appropriate for 
%   networks that spread information in a more diffusion-based manner 
%   (e.g., one axon can excite dendrites from multiple neurons). The Crofts 
%   & Higham adjustment for weighted networks is used.
%
%   Inputs:     W, weighted, undirected connection matrix - note that all
%                  weights should be between 0-1. 
%               
%   Output:     w, communicability betweenness for each node
%               
%   References: Estrada E, Higham DJ., Hatano N. (2009). Physica A: Stat Mech Appl, 388, 764-774.
%               Crofts JJ, Higham DJ. (2009). J R Soc Interface 6, 411-414.
%
%   Jeff Spielberg, University of Delaware
%   
%   Copyleft 2021-Forever. Software can be modified and redistributed, but
%   modified, redistributed versions must have the same rights

%   Modification History:
%   October 2021: Original


n  = size(W,1);               % Determine the number of nodes
W  = W.*(W>0).*(1-eye(n));    % Ensure that all weights are positive and the diagonal is cleared
S  = diag(sum(W));            % Compute the diagonal strength matrix
W  = (S^(-0.5))*W*(S^(-0.5)); % Apply Crofts and Higham adjustment
g1 = expm(W);                 % Compute communicability
C  = ((n-1).^2)-(n-1);        % Compute adjustment for number of possible links
w  = zeros(n,1);              % Preallocate space
for r = 1:n                   % For each node
    W0      = W;              % Set starting matrix
    W0(r,:) = 0;              % Remove current row
    W0(:,r) = 0;              % Remove current column
    g2      = expm(W0);       % Compute communicability of reduced network
    tp      = (g1-g2)./g1;    % Find differences in communicability
    tp(r,:) = 0;              % Set current row to 0
    tp(:,r) = 0;              % Set current column to 0
    tp      = tp.*(1-eye(n)); % Clear diagonal
    w(r)    = sum(sum(tp));   % Compute betweenness of current node
end
w = w./C;                     % Adjust for number of possible links
