function [mcd,tcd,cd] = comm_dist_u(W,de)

%   Communicability Distance
%
%   [mcd,tcd,cd] = comm_dist_u(W,de);
%
%   Mean communicability distance or efficiency for each node, across the 
%   network, and per connection. Communicability distance (inversely) 
%   reflects the overally quality of the communication between two nodes; 
%   that is, longer distances indicate worse communication. Distance here 
%   reflects the amount of wasted information sent (i.e., subgraph 
%   centrality, which is the amount of information emmitted by a node that 
%   is returned to that node) minus the amount of sucessfully communicated 
%   information (i.e., communicability between two nodes). Communicability 
%   distance can also be conceptualized as the euclidean distance between 
%   two nodes when they are embedded into a hypersphere. For ease of 
%   interpretation, we also include the option (called efficiency here) to 
%   reverse the scale, such that higher communicability efficiency is 
%   related to more efficient communication. For weighted networks, Crofts 
%   & Higham's adjustment is used.
%
%   Inputs:     W,    weighted/binary, undirected connection matrix
%               de,   use distance or efficiency (inverse distance)
%               
%   Output:     mcd,  mean communicability distance (or efficiency) for each node
%               tcd,  mean communicability distance (or efficiency) across the network
%               cd,   communicability distance (or efficiency)
%               
%   References: Estrada E, Hatano N. (2016). SIAM Rev 58, 692?715.
%               Crofts JJ, Higham DJ. (2009). J R Soc Interface 6, 411-414.
%
%
%   Jeff Spielberg, University of Delaware
%   
%   Copyleft 2016-Forever. Software can be modified and redistributed, but
%   modified, redistributed versions must have the same rights

%   Modification History:
%   December 2016: Original
%   March    2021: Added efficiency version

if ~exist('de','var')                  % If distance or efficiency not specified
    de = 1;                            % Default to distance
end
n = size(W,1);                         % Number of nodes
W = W.*(W>0).*(1-eye(n));              % Ensure that all weights are positive and the diagonal is cleared
S = diag(sum(W));                      % Compute the diagonal strength matrix
if length(unique(W))==2                % If matrix is binary
    G = expm(W);                       % Compute communicability
else                                   % If matrix is weighted
    G = expm((S^(-0.5))*W*(S^(-0.5))); % Compute communicability
end
sc  = diag(G);                         % Self-communicability
uno = ones(n,1);                       % Vector of ones
cd = sqrt((sc*uno')+(uno*sc')-(2.*G)); % Compute communicability distances
if de~='d' && de~=1
    cd = -cd;
end
mcd = cd*uno;                          % Compute mean communicability distance (or efficiency) per node
tcd = mcd'*uno;                        % Compute mean communicability distance (or efficiency) across network
