function [CC,CChar,SC,BC,eBC,ST,CPD,BK,BCe] = currflow_cent_wu(W,cr,prop1,ver)

%   Current-Flow Centralities
%
%   [CC,CChar,SC,BC,eBC,ST,CPD,BK,BCe] = currflow_cent_wu(W,cr,BC1,ver);
%
%   Computes several measures based on a current-flow model. Current-flow
%   networks take into account all possible paths between nodes, not just
%   the shortest. This may be more appropriate for networks that spread
%   information in a more diffusion-based manner (e.g., one axon can excite
%   dendrites from multiple neurons).
%
%   Two versions of closeness are computed, the first is traditional CF
%   closeness and second uses a harmonic mean. Note, traditional
%   current-flow closeness centrality is identical to information
%   centrality. Nodal spreading capacity is similar to closeness but
%   weighted differently (and is keyed in the opposite direction).
%
%   This version of node strength is the sum of the current flowing through
%   all of the links connected to a node (rather than the original link
%   weights).
%
%   Central point dominance reflects the dominance of the most central node
%   over all others.
%
%   This version of brokerage is a node-strength-weighted version of
%   current-flow betweenness centrality. This version differs from Everett
%   et al. in that it uses current-flow versions of betweenness and node
%   strength and allows weighted networks.
%
%   Bridging centrality measures the extent to which a node 'bridges'
%   densely connected regions.
%
%
%   Input:  W,     undirected, weighted connection matrix
%           cr,    flag indicating whether to use conductance (default) or
%                  resistance. For conductance specify 'c', for resistance
%                  specify 'r'.
%           prop1, flag to set other a property than CC as the first output
%                  argument. Options:
%                                   0 = CC (i.e., no change) (default)
%                                   1 = BC
%                                   2 = CChar
%                                   3 = SC
%                                   4 = eBC
%                                   5 = ST
%                                   6 = CPD
%                                   7 = BK
%                                   8 = BCe
%           ver,   Laplacian version to use (see laplace_mats.m for
%                  options; default is standard Laplacian)
%
%   Output: CC,    nodal current-flow closeness centrality (or other property if prop1 specified)
%           CChar, harmonic mean nodal current-flow closeness centrality
%           SC,    nodal spreading capacity
%           BC,    nodal current-flow betweenness centrality
%           eBC,   edge current-flow betweenness centrality
%           ST,    current-flow node strength
%           CPD,   central point dominance (of betweenness)
%           BK,    current-flow brokerage
%           BCe,   bridging centrality
%
%   References: Brandes U, Fleischer D. (2005) STACCS, 533-544
%               Newman MEJ. (2005). Soc Networks 27, 39-54.
%               Freeman LC. (1977). Sociometry 40, 35-41.
%               Everett M, Valente TW. (2016). Soc Networks 44, 202-208.
%               Hwang W, Cho Y, Zhang A, Ramanathan M. (2006). KDD'06, 20-23.
%               Wei K, Zeng, D, Guo S, Xu K. (2014). IEEE Transactions on Parallel and Distributed Systems 25, 1563-73.
%               Van Mieghem P, Devriendt K, Cetinay H. (2017). Phys Rev E, 96(3), 032311.
%
%   Jeff Spielberg, University of Delaware
%
%   Copyleft 2016-Forever. Software can be modified and redistributed, but
%   modified, redistributed versions must have the same rights

%   Modification History:
%   September 2016: Original
%   October   2016: Added CPD
%   April     2017: Added eBC
%   May       2017: Converted parts of computation into matrix form to
%                   increase computation speed (output identical to
%                   previous algorithm)
%   November  2017: Added check to determine what output has been requested
%                   before computing BC, etc. Added flag to set BC as the
%                   1st output argument
%   February  2018: Added option to use other versions of the Laplacian and
%                   current-flow brokerage, node strength, and bridging
%                   coefficient and centrality
%   May       2020: Edited for dramatic speed increases
%   August    2021: Changed BC1 to allow for all different outputs to be placed first

if ~exist('prop1','var') || isempty(prop1)                                                                        % If desired 1st output not input
    prop1 = 0;                                                                                                    % Default to no change
end
if ~exist('ver','var') || isempty(ver)                                                                            % If version not input
    ver = 'L';                                                                                                    % Default to Laplacian
end
n = size(W,1);                                                                                                    % Number of nodes
W = W.*(1-eye(n));                                                                                                % Ensure that the diagonal is cleared
if ~strcmp(ver,'sL') && ~strcmp(ver,'snsL') && ~strcmp(ver,'rnsL') && ~strcmp(ver,'sinsL') && ~strcmp(ver,'bnsL') % If signs are not to be retained
    W = W.*(W>0);                                                                                                 % Ensure that all weights are positive
end
if exist('cr','var') && ~isempty(cr) && cr=='r'                                                                   % If resistance is to be used
    W = weight_conversion(W,'lengths');                                                                           % Convert to lengths
end
Gp               = pinv(laplace_mats(W,ver));                                                                     % Moore-Penrose inverse of Laplacian
CC               = 1./(diag(Gp)+(trace(Gp)./n));                                                                  % Compute current-flow closeness centrality
CC(sum(W)==0)    = 0;                                                                                             % Remove influence of unreachable nodes
nmat             = diag(Gp)*ones(1,n);                                                                            % Create matrix of node components
ird              = n./(nmat+nmat'-(2*Gp));                                                                        % Compute inverse resistance distance
ird(isinf(ird))  = 0;                                                                                             % Remove infs due to unreachable nodes
CChar            = sum(ird,2)/((n-1)*(n-1));                                                                      % Compute harmonic mean current-flow closeness centrality
CChar(sum(W)==0) = 0;                                                                                             % Remove influence of unreachable nodes
SC               = diag(Gp);                                                                                      % Compute spreading capacity

if prop1==2                                                                                                       % Check whether CChar should be set as the first output argument
    CC = CChar;                                                                                                   % Set CChar to be the first output argument
elseif prop1==3                                                                                                   % Check whether SC should be set as the first output argument
    CC = SC;                                                                                                      % Set SC to be the first output argument
end

if nargout>3 || prop1~=0                                                                                          % If BC or more has been requested
    e = ones(n,1);                                                                                                % Create ones matrix
    E = eye(n);                                                                                                   % Create I matrix
    F = zeros(n,n);                                                                                               % Preallocate current flow for each link
    for i = 1:n                                                                                                   % For each node of interest
        M1 = Gp(:,i)-Gp(:,i)';                                                                                    % Calculate i contribution to current flow matrix
        for j = (i+1):n                                                                                           % For each potential node to pair with i
            M = abs(W(i,j)*(M1-Gp(:,j)+Gp(:,j)'));                                                                % Calculate full current flow matrix
            F(j,i) = (e-E(:,j))'*M*(e-E(:,j));                                                                    % Calculate current flow for this link
            F(i,j) = (e-E(:,i))'*M*(e-E(:,i));                                                                    % Calculate current flow for the converse link
        end
    end
    BC = sum(F,2)./((n-1)*(n-2)*2);                                                                               % Compute nodal betweenness by summing along all pairs with the node of interest
    if prop1==1                                                                                                   % Check whether BC should be set as the first output argument
        CC = BC;                                                                                                  % Set BC to be the first output argument
    end
    if nargout>4 || prop1>3                                                                                       % If BC has been requested
        eBC      = (F+F');                                                                                        % Compute edge betweenness by averaging across the two directions (i.e., i->j, j->i)
        ST(:,1)  = sum(F);                                                                                        % Compute node strength
        CPD      = sum(BC-max(BC))./(n-1);                                                                        % Compute central point dominance
        BK(:,1)  = BC.*(n-1).*(n-2).*2;                                                                           % Assign from node betweenness
        BK(BK>0) = (BK(BK>0)+n-1)./ST(BK>0);                                                                      % Compute brokerage
        if prop1==4                                                                                               % Check whether eBC should be set as the first output argument
            CC = eBC;                                                                                             % Set eBC to be the first output argument
        elseif prop1==5                                                                                           % Check whether ST should be set as the first output argument
            CC = ST;                                                                                              % Set ST to be the first output argument
        elseif prop1==6                                                                                           % Check whether CPD should be set as the first output argument
            CC = CPD;                                                                                             % Set CPD to be the first output argument
        elseif prop1==7                                                                                           % Check whether BK should be set as the first output argument
            CC = BK;                                                                                              % Set BK to be the first output argument
        end
        if nargout>8 || prop1==8                                                                                  % If BCe has been requested
            BCo = zeros(n,1);                                                                                     % Preallocate space
            iS  = 1./ST;                                                                                          % Invert node strength
            for cn = 1:n                                                                                          % For each node
                BCo(cn,1) = iS(cn)./sum(iS(W(:,cn)>0));                                                           % Compute bridging coefficient
            end
            BCo(iS==0) = 0;                                                                                       % Account for disconnected nodes, which will have a NaN denominator
            BCe(:,1)   = BCo.*BC;                                                                                 % Compute bridging centrality
            if prop1==8                                                                                           % Check whether BCe should be set as the first output argument
                CC = BCe;                                                                                         % Set BCe to be the first output argument
            end
        end
    end
end
