function C = immed_effects_cent_u(A)

%   Immediate Effects Centrality
%
%   C = immed_effects_cent_u(A);
%
%   Immediate effects centrality reflects the extent to which a node's
%   influence on other nodes is transmitted, on average, via longer or
%   shorter sequences. In other words, how immediate are the effects of a
%   node.
%
%   Input:  A,     undirected, weighted/binary connection matrix
%
%   Output: C,     immediate effects centrality
%
%   Reference: Friedkin NE. (1991), Amer J Sociol 96, 1478-1504.
%
%
%   Jeff Spielberg, University of Delaware

%   Modification History:
%   October 2016: Original

n  = size(A,1);                                        % Number of nodes
A  = A.*(A>0).*(1-eye(n));                             % Ensure that all weights are positive and the diagonal is cleared
Tj = zeros(n,n,n);                                     % Preallocate space
singmatwarn(2)                                         % Set behavior of warnings related to singular matrices to behave like errors
try                                                    % Try to compute C
    W  = A./repmat(sum(A,2),1,n);                      % Compute influence network
    for j = 1:n
        Wj                    = W;                     % Set current influence matrix
        Wj(j,[1:j-1,j+1:end]) = 0;                     % Set current column to 0
        Wj([1:j-1,j+1:end],j) = 0;                     % Set current row to 0
        Tj(:,:,j)             = inv(eye(n)-Wj);        % Compute the number of sequences from node j to node i via each node k
    end
    Tj(1:(n*(n+1))+1:end) = 0;                         % Set global self connections to 0
    M                     = squeeze(sum(Tj,2));        % Compute mean length of inter-node influence
    C                     = ((sum(M,1)/(n-1)).^(-1))'; % Compute immediate effects centrality
catch                                                  % If warning found
    C = NaN(n,1);                                      % Set output to NaN
end
singmatwarn(1)                                         % Reset behavior of warnings