function mat = laplace_mats(W,ver,a)

%   mat = laplace_mats(W,ver,a)
%
%   Computes several versions of the Laplacian matrix, along with related 
%   matrices. 
%
%   Input:  W,   undirected binary or weighted, positive connection matrix
%           ver, specify the desired matrix:
%                   'L'     = regular old Laplacian
%                   'snL'   = symmetric normalized Laplacian
%                   'rnL'   = random-walk normalized Laplacian
%                   'naL'   = normalized additive Laplacian
%                   'slL'   = signless Laplacian
%                   'sL'    = signed Laplacian
%                   'snsL'  = symmetric normalized signed Laplacian
%                   'rnsL'  = random-walk normalized signed Laplacian
%                   'sinsL' = simple normalized signed Laplacian
%                   'bnsL'  = balanced normalized signed Laplacian
%                   'dL'    = deformed Laplacian
%                   'mt'    = Markov transition matrix
%                   'gm'    = "google" matrix
%           a,   adjustment value needed for some matrices
%
%   Output: mat,  requested matrix
% 
% References:
%           Laplacian, Symmetric normalized Laplacian: Butler S. (2007). ELA 16, 90-98.
%           Normalized additive Laplacian: Kamvar SD, Klein D, Manning CD. (2003).
%           Signless Laplacian: Zhu P, Wilson RC. (2005). BMVC.
%           Random-walk normalized Laplacian, signed Laplacian, symmetric normalized signed Laplacian, random-walk normalized signed Laplacian, Markov transition matrix: Kunegis J, Schmidt S, Lommatzsch A, Lerner J, De Luca EW, Albayrak S. (2010). SDM 10, 559-559.
%           Simple normalized signed Laplacian, balanced normalized signed Laplacian: Zheng Q, Skillicorn DB. (2015). SIAM international conference on data mining, 55-63.
%           Deformed Laplacian: Morbidi F. (2013). Automatica 49, 3049-55. 
%           Google matrix: Brin S, Page L. (1998). Computer Networks and ISDN Systems 30, 107.
%
%   Jeff Spielberg, University of Delaware
%   
%   Copyleft 2021-Forever. Software can be modified and redistributed, but
%   modified, redistributed versions must have the same rights

%   Modification History:
%   October   2021: Published version

if ~exist('ver','var')                       % If version not input
    ver = 'L';                               % Default to regular old Laplacian
end
if ~exist('a','var') && strcmp(ver,'gm')     % If weight not input and google matrix requested
    a = 0.85;                                % Default to 0.85
elseif ~exist('a','var') && strcmp(ver,'dL') % If weight not input and deformed Laplacian requested
    a = 2;                                   % Default to 2
end
n            = size(W,1);                    % Determine number of nodes
W(1:n+1:end) = 0;                            % Remove self-connections
S            = diag(sum(W,1));               % Compute diagonalized node strength matrix

% Compute matrices
switch ver
    case 'L'
        mat = S-W;                                    % Compute Laplacian
    case 'snL'
        mat = sqrt(S)\(S-W)/sqrt(S);                  % Compute symmetric normalized Laplacian
    case 'rnL'
        mat = S\(S-W);                                % Compute random-walk normalized Laplacian
    case 'naL'
        mat = ((max(S(:)).*eye(n))-(S-W))./max(S(:)); % Compute normalized additive Laplacian
    case 'slL'
        mat = S+W;                                    % Compute signless Laplacian
    case 'sL'
        mat = diag(sum(abs(W),1))-W;                  % Compute signed Laplacian
    case 'snsL'
        S   = diag(sum(abs(W),1));                    % Compute absolute valued node strength matrix
        mat = sqrt(S)\(S-W)/sqrt(S);                  % Compute symmetric normalized signed Laplacian
    case 'rnsL'
        S   = diag(sum(abs(W),1));                    % Compute absolute valued node strength matrix
        mat = eye(n)-(S\W);                           % Compute random-walk normalized signed Laplacian
    case 'sinsL'
        Sp  = diag(sum(W.*(W>0),1));                  % Compute positive node strength matrix
        Sn  = diag(sum(-W.*(W<0),1));                 % Compute negative node strength matrix
        Sm  = Sp+Sn;                                  % Compute absolute valued node strength matrix
        mat = Sm\(Sp-Sn-W);                           % Compute simple normalized signed Laplacian
    case 'bnsL'
        Sp  = diag(sum(W.*(W>0),1));                  % Compute positive node strength matrix
        Sn  = diag(sum(-W.*(W<0),1));                 % Compute negative node strength matrix
        Sm  = Sp+Sn;                                  % Compute absolute valued node strength matrix
        mat = Sm\(Sp-W);                              % Compute simple normalized signed Laplacian
    case 'dL'
        mat = eye(n)-(a.*W)+((S-eye(n))*(a.^2));      % Compute deformed Laplacian
    case 'mt'
        mat = S\W;                                    % Compute Markov transition matrix (otherwise known as the 'normal' matrix)
%         mat = W/S;                                  % Alternate formula for Markov transition matrix
    case 'gm'
        mat = (a.*(S-W))+((1-a)./n);                  % Compute Google matrix
end
