function C = med_effects_cent_u(A)

%   Mediative Effects Centrality
%
%   C = med_effects_cent_u(A);
%
%   Mediative effects centrality reflects the extent to which a node is
%   involved in transmitting the effects of other nodes. 
%
%   Input:  A,     undirected, weighted/binary connection matrix
%
%   Output: C,     mediative effects centrality
%
%   Reference: Friedkin NE. (1991), Amer J Sociol 96, 1478-1504.
%
%
%   Jeff Spielberg, University of Delaware

%   Modification History:
%   October 2016: Original

n   = size(A,1);                                       % Number of nodes
A   = A.*(A>0).*(1-eye(n));                            % Ensure that all weights are positive and the diagonal is cleared
Tk  = zeros(n,n,n);                                    % Preallocate space
mTk = zeros(n,n);                                      % Preallocate space
ind = 1:n;                                             % Set possible indices
singmatwarn(2)                                         % Set behavior of warnings related to singular matrices to behave like errors
try                                                    % Try to compute C
    W = A./repmat(sum(A,2),1,n);                       % Compute influence network
    for k = 1:n
        Wk                    = W;                     % Set current influence matrix
        Wk(k,[1:k-1,k+1:end]) = 0;                     % Set current column to 0
        Wk([1:k-1,k+1:end],k) = 0;                     % Set current row to 0
        Tk(:,:,k)             = inv(eye(n)-Wk);        
    end
    Tk(1:(n*(n+1))+1:end) = 0;                         % Set global self connections to 0
    for j = 1:n
        for k = 1:n
            if j~=k                                    % Ignore self-connections
                mTk(j,k) = sum(Tk(ind((ind~=j)&(ind~=k)),j,k),1)./((n-2)*Tk(j,j,k));
            end
        end
    end
    C = sum(mTk,2)/(n-1)';                             % Compute mediative effects centrality
catch                                                  % If warning found
    C = NaN(n,1);                                      % Set output to NaN
end
singmatwarn(1)                                         % Reset behavior of warnings
