function [C,Ctot,SC,SCtot] = node_communicability_wu(W,Ltype,sinc)

%   Node Communicability
%
%   [C,Ctot,SC,SCtot] = node_communicability_wu(W,Ltype,sinc);
%
%   Sum of Estrada's communicability matrix for each node using Crofts &
%   Higham's adjustment for weighted networks. Communicability reflects the
%   quality of communication between a node and all other nodes, along all
%   paths. Communicability can be computed based on either the original
%   connectivity matrix or the Laplacian. Another option is whether or not
%   to include subgraph centrality (i.e., diagonal of the communicability
%   matrix), which reflects the quality of communication of a node with
%   itself (or, how much a node dominates the network, and thus its
%   information dominates the network and is returned to the node). Benzi
%   and Klymko (2013) include subgraph centralities in their version,
%   whereas Estrada and Arrigo (2015) do not (at least with regard to the
%   total communicability of the entire network). 
%
%   Inputs:     W,        weighted, undirected connection matrix
%               Ltype,    matrix adjustment; to perform no adjustment,
%                         leave blank or enter 'adj'; otherwise, can use
%                         any type available in laplace_mats.m
%               sinc,     flag for whether to subtract subgraph centrality
%                         (i.e., self-communication) in the communicability
%                         measures. Specify 0 to include (default), 1 to
%                         subtract
%
%   Output:     C,        communicability for each node
%               Ctot,     total communicability across the network
%               SC,       subgraph centrality
%               SCtot,    total subgraph centrality across the network
%               
%   References: Benzi M, Klymko C. (2013). J Comp Net 1, 124-149.
%               Crofts JJ, Higham DJ. (2009). J R Soc Interface 6, 411-414.
%               Estrada E, Arrigo F. (2015). SIAM J App Math 75, 1725-1744.
%               Estrada E, Hatano N. (2008). Phys Rev E 77, 036111.
%               Estrada E, Hatano N, Benzi M. (2012). Phys Rep, 514, 89-119.
%
%   Jeff Spielberg, University of Delaware
%   
%   Copyleft 2016-Forever. Software can be modified and redistributed, but
%   modified, redistributed versions must have the same rights

%   Modification History:
%   April 2016: Original
%   January 2018: Added output of subgraph centrality

n = size(W,1);                                                     % Number of nodes
W = W.*(W>0).*(1-eye(n));                                          % Ensure that all weights are positive and the diagonal is cleared
S = diag(sum(W));                                                  % Compute the diagonal strength matrix
if ~exist('Ltype','var') || isempty(Ltype) || strcmpi(Ltype,'adj') % If no adjustment specified
else                                                               % Use Laplacian
    W = laplace_mats(W,Ltype);                                     % Compute matrix
end
G = expm((S^(-0.5))*W*(S^(-0.5)));                                 % Compute communicability
if nargout>2                                                       % If subgraph centrality requested
    SC    = diag(G);                                               % Compute subgraph centrality
    SCtot = sum(SC);                                               % Compute total subgraph
end
if exist('sinc','var') && sinc==1                                  % If subgraph centrality is to be removed
    G = G.*(1-eye(n));                                             % Remove subgraph centrality (i.e., the diagonal)
end
C    = G*ones(n,1);                                                % Compute node communicability
Ctot = sum(C);                                                     % Compute total communicability
