function [NIC,rnNIC,cnNIC,rcnNIC,crnNIC,rmcNIC,cmcNIC] = node_indu_cent_wu(W,metric,type,sr)

%   Node Induced Centrality
%
%   [NIC,rmcNIC,cmcNIC,rnNIC,cnNIC,rcnNIC,crnNIC] = node_indu_cent_wu(W,metric,type,sr);
%
%   Computes node induced centrality for a variety of metrics. Node induced
%   centrality for node X reflects the impact on node X's centrality of
%   removing node Y from the network. In other words, how important is node
%   Y for node X to function effectively. For more information, see:
%   
%   Matyi MA, Cioaba S, Banich MT, & Spielberg JM. (2021). Identifying 
%       brain regions supporting amygdalar functionality: Application of a 
%       novel graph theory technique. NeuroImage, 244, 118614. 
%       doi:10.1016/j.neuroimage.2021.118614
%
%   Note that more properties are available in this script than were 
%   examined in the publication above - use special care when using these.
%   
%   Node 'removal': There are three methods by which nodes can be "removed"
%                   from the network. (i) actually remove node Y and 
%                   examine the reduced network; (ii) set all of node Y's
%                   links to zero; (iii) set all of node Y's links to the
%                   smallest possible number in matlab. The potential 
%                   benefit of methods (ii) and (iii) over (i) is that they
%                   remove the impact of node Y but maintain the same 
%                   number of nodes. The potential benefit of method (iii)
%                   over (ii) is that the "removed" node will not be 
%                   disconnected. Thus, only method (iii) has been used in 
%                   publication thus far and is the default. 
%   
%   Output interpretation: The main output is coded such that positive 
%                   values indicate that centrality for node X *increased* 
%                   when node Y was removed (i.e., X became more central to
%                   the network), whereas negative values indicate that 
%                   node X centrality *decreased* when node Y was removed 
%                   (i.e., X became less central). Note, this is only true 
%                   for properties where larger values indicate greater 
%                   centrality - the opposite interpretation is applicable
%                   for properties where greater centrality is indicated by
%                   smaller postive or larger negative values. 
%   
%   Note, this function can be time intensive, and computation time
%   increases exponentially with matrix size.
%   
%   Input:  W,      undirected binary or weighted, positive connection matrix
%           metric, specify the desired metric:
%                       'cf_closecent' = current flow closeness centrality
%                       'cf_betcent'   = current flow betweenness centrality
%                       'cf_sprcap'    = current flow spreading capacity
%                       'cf_brok'      = current flow brokerage
%                       'cf_brdgcent'  = current flow bridging centrality
%                       'eigcent'      = eigenvector centrality
%                       'commdist'     = communicability distance
%                       'nodecomm'     = node communicability
%                       'commbet'      = communicability betweenness
%                       'subgraph'     = subgraph centrality
%                       'clustcoef'    = clustering coefficient
%                       'immeffcent'   = immediate effects centrality
%                       'medeffcent'   = mediative effects centrality
%                       'toteffcent'   = total effects centrality
%                       'strength'     = node strength (will only show
%                                        nodes with strong direct
%                                        connections as important, thus may
%                                        be used as a sanity check)
%
%          type,    method of by which the node is "removed":
%                       1 = physically remove the node and recompute on the
%                           smaller network
%                       2 = keep the "removed" node in the network, but set
%                           it's links to 0
%                       3 = keep the "removed" node in the network, but set
%                           it's non-zero links to the smallest number
%                           possible in matlab (default)
%          sr,      controls how the original and 'removed' values are compared
%                       's' = subtract original from 'reduced' (default)
%                       'r' = divide 'reduced' by original
%
%   Output: NIC,    n*n matrix of node induced centralities (NIC), where
%                   each row contains the centralities for the node
%                   corresponding to that row and each column reflects the
%                   impact of removing the node corresponding to that
%                   column. For example, the value in row 2, column 5 is
%                   the impact of removing node 5 on the centrality of node
%                   2.
%           rmcNIC, n*n matrix of NICs that have been mean centered by row
%                   (i.e., mean across the row is 0). Thus, deviations from
%                   0 correspond to changes relative to the average removed
%                   node (e.g., a negative value indicates that removing
%                   this node decreases the centrality of the target node
%                   more than the average node removal). 
%           cmcNIC, n*n matrix of NICs that have been mean centered by
%                   column (i.e., mean across the column is 0). Thus,
%                   deviations from 0 correspond to changes relative  to
%                   the average target node (e.g., a negative value
%                   indicates that the removed node decreases the
%                   centrality of this target node more than the average
%                   target node). 
%           rnNIC,  n*n matrix of NICs that have been normalized (divided)
%                   by the (absolute value of the) mean NIC for that row.
%                   This is to boost the values of those ROIs whose
%                   removal is *relatively* important for the target node
%                   and suppress ROIs that are *relatively* unimportant.
%                   The absolute value of the mean is used so that the
%                   direction of effects is not changed. Values are set
%                   such that the mean across the row is 0, and thus
%                   deviations from 0 correspond to changes relative to the
%                   average removed node (e.g., a negative value indicates
%                   that removing this node decreases the centrality of the
%                   target node more than the average node removal). 
%           cnNIC,  n*n matrix of NICs that have been normalized by the
%                   (absolute value of the) mean NIC for that column. This
%                   is to account for the fact that the removal of some
%                   nodes may have a high impact on most other nodes, and
%                   thus are not uniquely important for any specific node.
%                   This normalization accounts for this issue, suppressing
%                   NICs corresponding to universally important nodes
%                   and boosting NICs corresponding to nodes with low
%                   overall impact. The absolute value of the mean is used
%                   so that the direction of effects is not changed. Values
%                   are set such that the mean across the column is 0, and
%                   thus deviations from 0 correspond to changes relative 
%                   to the average target node (e.g., a negative value
%                   indicates that the removed node decreases the
%                   centrality of this target node more than the average
%                   target node). 
%           rcnNIC, n*n matrix of NICs that have been 1st normalized
%                   (divided) by the mean NIC for that row, and 2nd by the
%                   mean NIC for that column. Values are set such that the
%                   mean across the column is 0, and thus deviations from 0
%                   correspond to changes relative to the average target
%                   node (e.g., a negative value indicates that the removed
%                   node decreases the centrality of this target node more
%                   than the average target node). 
%           crnNIC, n*n matrix of NICs that have been 1st normalized
%                   (divided) by the mean NIC for that column, and 2nd by
%                   the mean NIC for that row. Values are set such that the
%                   mean across the row is 0, and so deviations from 0
%                   correspond to changes relative to the average removed
%                   node (e.g., a negative value indicates that removing
%                   this node decreases the centrality of the target node
%                   more than the average node removal). These values may
%                   appear redundant with rcnNIC, but they differ in terms
%                   of what deviations from 0 correspond to in particular. 
%
%   Dependencies:   
%       clustering_coef_wu_sign.m
%       comm_bet_cent_wu.m
%       comm_dist_u.m
%       currflow_cent_wu.m
%       immed_effects_cent_u.m
%       laplace_mats.m
%       med_effects_cent_u.m
%       node_communicability_wu.m
%       spec_cent_u.m
%       subgraph_centrality_wu.m
%       tot_effects_cent_u.m
%       Brain Connectivity Toolbox
%
%
%   Jeff Spielberg, University of Delaware
%   
%   Copyleft 2020-Forever. Software can be modified and redistributed, but
%   modified, redistributed versions must have the same rights

%   Modification History:
%   January   2020: Original submitted version
%   August    2021: Edited to include current flow spreading capacity, current flow brokerage, current flow bridging centrality, communicability betweenness, immediate effects centrality, mediative effects centrality, total effects centrality
%   October   2021: Published version

% Ensure that a metric has been specifed and set default value for 'type' if unspecified
if nargin==1
    error('Specify a metric')
elseif nargin==2
    type = 3;
end

n   = size(W,1);            % Determine number of nodes
W   = W.*(W>0).*(1-eye(n)); % Remove self connections
NIC = nan(n,n);             % Preallocate space

% Set default values
exarg = '';                 % Extra arguments
f     = 0;                  % Full network value
r     = 0;                  % Reduced network value
if ~exist('sr','var')
    sr = 's';               % Set default to subtract reduced value
end

% Determine which metric is desired, and set appropriate options
switch metric
    case 'cf_closecent' % Current-flow closeness centrality
        funcname = 'currflow_cent_wu';
    case 'cf_betcent'   % Current-flow node betweenness centrality
        funcname = 'currflow_cent_wu';
        exarg    = ',''c'',1';
    case 'cf_sprcap'    % Current-flow spreading capacity
        funcname = 'currflow_cent_wu';
        exarg    = ',''c'',3';
    case 'cf_brok'      % Current-flow node brokerage
        funcname = 'currflow_cent_wu';
        exarg    = ',''c'',7';
    case 'cf_brdgcent'  % Current-flow node bridging centrality
        funcname = 'currflow_cent_wu';
        exarg    = ',''c'',8';
    case 'eigcent'      % Traditional eigenvector centrality
        funcname = 'spec_cent_u';
        exarg    = ',''adj'',2';
    case 'commdist'     % Communicability distance
        funcname = 'comm_dist_u';
    case 'nodecomm'     % Node communicability
        funcname = 'node_communicability_wu';
        exarg    = ',''adj'',1';
    case 'commbet'      % Communicability betweenness
        funcname = 'comm_bet_cent_wu';
    case 'subgraph'     % Subgraph centrality
        funcname = 'subgraph_centrality_wu';
    case 'clustcoef'    % Clustering coefficient
        funcname = 'clustering_coef_wu_sign';
        exarg    = ',2';
    case 'immeffcent'   % Immediate effects centrality
        funcname = 'immed_effects_cent_u';
    case 'medeffcent'   % Mediative effects centrality
        funcname = 'med_effects_cent_u';
    case 'toteffcent'   % Total effects centrality
        funcname = 'tot_effects_cent_u';
    case 'strength'     % Node strength
        funcname = 'strengths_und';
    otherwise
        error('Undefined metric')
end

if type==1                                                                            % Remove node and recompute
    eval(['f = ',funcname,'(W',exarg,');']);                                          % Compute property on original matrix
    for cn = 1:n                                                                      % For each 'removed' node
        eval(['r = ',funcname,'(W([1:cn-1,cn+1:end],[1:cn-1,cn+1:end])',exarg,');']); % Remove row/column corresponding to current removed node, then compute property on reduced network
        if sr=='s'                                                                    % If subtraction is to be used
            NIC(:,cn) = [r(1:cn-1);0;r(cn:end)]-f;                                    % Compute difference between new and old values
        elseif sr=='r'                                                                % If the ratio is to be used
            NIC(:,cn) = [r(1:cn-1);0;r(cn:end)]./f;                                   % Compute ratio of the new to old values
        end
        NIC(cn,cn) = 0;                                                               % Set self-impact to 0
    end
elseif type==2                                    % Set connections for 'removed' node to 0
    eval(['f = ',funcname,'(W',exarg,');']);      % Compute property for original network
    for cn = 1:n                                  % For each 'removed' node
        Wr         = W;                           % Reset matrix to original state
        Wr(cn,:)   = 0;                           % Set all links for current 'removed' node to 0
        Wr(:,cn)   = 0; %#ok<*NASGU>              % Set all links for current 'removed' node to 0
        eval(['r = ',funcname,'(Wr',exarg,');']); % Compute property on new network
        if sr=='s'                                % If subtraction is to be used
            NIC(:,cn) = r-f;                      % Compute difference between new and old values
        elseif sr=='r'                            % If the ratio is to be used
            NIC(:,cn) = r./f;                     % Compute the ratio of the new to old values
        end
        NIC(cn,cn) = 0;                           % Set self-impact to 0
    end
elseif type==3                                    % Option to set present connections for 'removed' node to the smallest number possible
    eval(['f = ',funcname,'(W',exarg,');']);      % Compute property for original network
    for cn = 1:n                                  % For each 'removed' node
        Wr                = W;                    % Reset matrix to original state
        Wr(cn,W(:,cn)~=0) = realmin;              % Set non-zero links for current 'removed' node to the smallest number allowed in matlab
        Wr(W(cn,:)~=0,cn) = realmin;              % Set non-zero links for current 'removed' node to the smallest number allowed in matlab
        eval(['r = ',funcname,'(Wr',exarg,');']); % Compute property on new network
        if sr=='s'                                % If subtraction is to be used
            NIC(:,cn) = r-f;                      % Compute difference between new and old values
        elseif sr=='r'                            % If the ratio is to be used
            NIC(:,cn) = r./f;                     % Compute the ratio of the new to old values
        end
        NIC(cn,cn) = 0;                           % Set self-impact to 0
    end
end

% Create row or column mean centered versions
rmcNIC         = NIC-repmat(mean(deldiag(NIC,1),2),1,n);                                            % Mean center by row, such that deviations from 0 reflect changes from the average for that row
rmcNIC(NIC==0) = 0;                                                                                 % Reset self-connections to 0
cmcNIC         = NIC-repmat(mean(deldiag(NIC,2),1),n,1);                                            % Mean center by column, such that deviations from 0 reflect changes from the average for that column
cmcNIC(NIC==0) = 0;                                                                                 % Reset self-connections to 0

% Create normalized (and mean centered) versions
rnNIC          = (NIC./abs(repmat(mean(deldiag(NIC,1),2),1,n)))-repmat(sign(sum(NIC,2)),1,n);       % Normalize by row mean, then mean center (by row) such that deviations from 0 reflect changes from the average for that row
rnNIC(NIC==0)  = 0;                                                                                 % Reset self-connections to 0
cnNIC          = (NIC./abs(repmat(mean(deldiag(NIC,2),1),n,1)))-repmat(sign(sum(NIC,1)),n,1);       % Normalize by column mean, then mean center (by column) such that deviations from 0 reflect changes from the average for that column
cnNIC(NIC==0)  = 0;                                                                                 % Reset self-connections to 0
rcnNIC         = (rnNIC./abs(repmat(mean(deldiag(rnNIC,2),1),n,1)))-repmat(sign(sum(rnNIC,1)),n,1); % Normalize first by row mean, then by column, then mean center (by column) such that deviations from 0 reflect changes from the average for that column
rcnNIC(NIC==0) = 0;                                                                                 % Reset self-connections to 0
crnNIC         = (cnNIC./abs(repmat(mean(deldiag(cnNIC,1),2),1,n)))-repmat(sign(sum(cnNIC,2)),1,n); % Normalize first by column mean, then by row, then mean center (by row) such that deviations from 0 reflect changes from the average for that row
crnNIC(NIC==0) = 0;                                                                                 % Reset self-connections to 0
