function [specc,wasdisc] = spec_cent_u(W,mattype,discalg,alpha,combmat)

%   Spectral Centralities
%
%   [specc,wasdisc] = spec_cent_u(W,mattype,discalg,alpha,combmat);
%
%   Compute several types of spectral (i.e., eigenvector) centrality.
%
%   Input:  W,       undirected, weighted/binary connection matrix
%           mattype, type of matrix from which to calculate eigenvectors
%                    'adj'   = unaltered connection matrix (default)
%                    'L'     = Laplacian of connection matrix
%                    'wcomb' = weighted combination of adjacency and
%                              user-supplied matrices (weighted via alpha).
%                              Note: computation of wcomb ignores
%                              discalg and computes as if no disconnected
%                              matrices are present (because, the supplied
%                              matrix might change when formed into
%                              submatrices).
%           discalg, what to do with disconnected matrices
%                    1 = nothing different (default)
%                    2 = compute centrality separately for each component,
%                        weighting each by the percentage of nodes in that
%                        component
%                    3 = compute centrality separately for each component,
%                        weighting each by the percentage of nodes in that
%                        component, adjusted by the number of components.
%                        This adjustment has the effect of overweighting
%                        relatively large components, and the extent of
%                        overweighting increases (exponentially) with the
%                        number of components. One important result is
%                        that, in the extreme case where all but one
%                        component contain only a single node, the output
%                        value is driven completely by the single large
%                        component.
%                    4 = compute centrality separately for each component,
%                        no weighting is applied
%           alpha,   weight for combination mattypes; must be between 0 and
%                    1, inclusive; values closer to 1 place more weight
%                    on the adjacency matrix, whereas values closer to 0
%                    weight the second matrix more highly (default = 0.5)
%           combmat, matrix to combine with adjacency matrix (default =
%                    diagonal strength matrix)
%
%   Output: specc,   spectral centrality
%           wasdisc, indicator of whether the network was disconnected
%
%   Output: specc,   centrality
%
%   References: Chung FR. (1997). Spectral graph theory. Am Math Soc.
%               Kunegis J, Schmidt S, Lommatzsch A, Lerner J, De Luca EW, Albayrak S. (2010). SDM 10, 559-559.
%
%
%   Jeff Spielberg, University of Delaware
%   Sebi Cioaba, University of Delaware
%   
%   Copyleft 2016-Forever. Software can be modified and redistributed, but
%   modified, redistributed versions must have the same rights

%   Modification History:
%   October 2016: Original

if ~exist('mattype','var')                                                                       % If matrix version is not input
    mattype = 'adj';                                                                             % Default to original eigenvector centrality
end
if ~exist('discalg','var') || strcmp(mattype,'wcomb')                                            % If algorithm to use in case of disconnected graphs is not input or if weighted combination is requested
    discalg = 1;                                                                                 % Default to no adjustment
end
n               = size(W,1);                                                                     % Number of nodes
tol             = 10^(-3);                                                                       % Tolerance for eigenvalues
W               = W.*(1-eye(n));                                                                 % Ensure that the diagonal is cleared
specc           = nan(n,1);                                                                      % Preallocate space
[comps,comp_ns] = get_components(W);                                                             % Find connected components
if length(comp_ns)>1                                                                             % Determine if network is disconnected
    wasdisc = 1;
else
    wasdisc = 0;
end
switch mattype                                                                                   % Type of matrix to use
    case {'adj'}                                                                                 % Matrix to use
        if discalg==1                                                                            % No adjustment for disconnected matrices
            L             = W;                                                                               % Make no adjustment
            [v,d]         = eig(L,'vector');                                                     % Compute eigenvectors and values
            d(abs(d)<tol) = nan;                                                                 % Set 0 eigenvalues to nan
            [~,ind]       = max(d);                                                              % Find maximum eigenvalue
            specc(:,1)    = abs(v(:,ind));                                                       % Centrality is associated eigenvector
        elseif discalg==2                                                                        % Compute for each component, then weight by the proportion of nodes contained in this component (used by ORA software)
            for c = 1:length(comp_ns)                                                            % For each component
                if comp_ns(c)>1                                                                  % If there is more than one node in the current component
                    Wc = W(comps==c,comps==c);                                                   % Create matrix containing only current component
                    L = Wc;                                                                      % Make no adjustment
                    [v,d]             = eig(L,'vector');                                         % Compute eigenvectors and values
                    d(abs(d)<tol)     = nan;                                                     % Set 0 eigenvalues to nan
                    [~,ind]           = max(d);                                                  % Find maximum eigenvalue
                    specc(comps==c,1) = abs(v(:,ind)).*comp_ns(c)./n;                            % Centrality is associated eigenvector
                else                                                                             % If there is only one node in the current component
                    specc(comps==c,1) = 0;                                                       % Set centrality to 0
                end
            end
        elseif discalg==3                                                                        % Compute for each component, then weight by the proportion of nodes contained in this component, adjusted by the number of components (used by visone software)
            for c = 1:length(comp_ns)                                                            % For each component
                if comp_ns(c)>1                                                                  % If there is more than one node in the current component
                    Wc = W(comps==c,comps==c);                                                   % Create matrix containing only current component
                    L = Wc;                                                                      % Make no adjustment
                    [v,d]             = eig(L,'vector');                                         % Compute eigenvectors and values
                    d(abs(d)<tol)     = nan;                                                     % Set 0 eigenvalues to nan
                    [~,ind]           = max(d);                                                  % Find maximum eigenvalue
                    specc(comps==c,1) = abs(v(:,ind)).*(comp_ns(c)-1)./(n-length(comp_ns));      % Centrality is associated eigenvector
                else                                                                             % If there is only one node in the current component
                    specc(comps==c,1) = 0;                                                       % Set centrality to 0
                end
            end
        elseif discalg==4                                                                        % Compute for each component, no weighting
            for c = 1:length(comp_ns)                                                            % For each component
                if comp_ns(c)>1                                                                  % If there is more than one node in the current component
                    Wc = W(comps==c,comps==c);                                                   % Create matrix containing only current component
                    L = Wc;                                                                      % Make no adjustment
                    [v,d]             = eig(L,'vector');                                         % Compute eigenvectors and values
                    d(abs(d)<tol)     = nan;                                                     % Set 0 eigenvalues to nan
                    [~,ind]           = max(d);                                                  % Find maximum eigenvalue
                    specc(comps==c,1) = abs(v(:,ind));                                           % Centrality is associated eigenvector
                else                                                                             % If there is only one node in the current component
                    specc(comps==c,1) = 0;                                                       % Set centrality to 0
                end
            end
        end
    case {'L'}                                                                                   % Use Laplacian type matrix
        if discalg==1                                                                            % No adjustment for disconnected matrices
            L             = laplace_mats(W,mattype);                                             % Obtain matrix
            [v,d]         = eig(L,'vector');                                                     % Compute eigenvectors and values
            d(abs(d)<tol) = nan;                                                                 % Set 0 eigenvalues to nan
            [~,ind]       = min(d);                                                              % Find minimum eigenvalue
            specc(:,1)    = abs(v(:,ind));                                                       % Centrality is associated eigenvector
        elseif discalg==2                                                                        % Compute for each component, then weight by the proportion of nodes contained in this component (used by ORA software)
            for c = 1:length(comp_ns)                                                            % For each component
                if comp_ns(c)>1                                                                  % If there is more than one node in the current component
                    Wc                = W(comps==c,comps==c);                                    % Create matrix containing only current component
                    L                 = laplace_mats(Wc,mattype);                                % Obtain matrix
                    [v,d]             = eig(L,'vector');                                         % Compute eigenvectors and values
                    d(abs(d)<tol)     = nan;                                                     % Set 0 eigenvalues to nan
                    [~,ind]           = min(d);                                                  % Find minimum eigenvalue
                    specc(comps==c,1) = abs(v(:,ind)).*comp_ns(c)./n;                            % Centrality is associated eigenvector
                else                                                                             % If there is only one node in the current component
                    specc(comps==c,1) = 0;                                                       % Set centrality to 0
                end
            end
        elseif discalg==3                                                                        % Compute for each component, then weight by the proportion of nodes contained in this component, adjusted by the number of components (used by visone software)
            for c = 1:length(comp_ns)                                                            % For each component
                if comp_ns(c)>1                                                                  % If there is more than one node in the current component
                    Wc                = W(comps==c,comps==c);                                    % Create matrix containing only current component
                    L                 = laplace_mats(Wc,mattype);                                % Obtain matrix
                    [v,d]             = eig(L,'vector');                                         % Compute eigenvectors and values
                    d(abs(d)<tol)     = nan;                                                     % Set 0 eigenvalues to nan
                    [~,ind]           = min(d);                                                  % Find minimum eigenvalue
                    specc(comps==c,1) = abs(v(:,ind)).*(comp_ns(c)-1)./(n-length(comp_ns));      % Centrality is associated eigenvector
                else                                                                             % If there is only one node in the current component
                    specc(comps==c,1) = 0;                                                       % Set centrality to 0
                end
            end
        elseif discalg==4                                                                        % Compute for each component, no weighting
            for c = 1:length(comp_ns)                                                            % For each component
                if comp_ns(c)>1                                                                  % If there is more than one node in the current component
                    Wc                = W(comps==c,comps==c);                                    % Create matrix containing only current component
                    L                 = laplace_mats(Wc,mattype);                                % Obtain matrix
                    [v,d]             = eig(L,'vector');                                         % Compute eigenvectors and values
                    d(abs(d)<tol)     = nan;                                                     % Set 0 eigenvalues to nan
                    [~,ind]           = min(d);                                                  % Find minimum eigenvalue
                    specc(comps==c,1) = abs(v(:,ind));                                           % Centrality is associated eigenvector
                else                                                                             % If there is only one node in the current component
                    specc(comps==c,1) = 0;                                                       % Set centrality to 0
                end
            end
        end
    case 'wcomb'                                                                                 % Use weighted combination of adjancency and user-suplied matrix
        if ~exist('combmat','var')                                                               % If weight not input
            combmat = diag(sum(W));                                                              % Compute diagonal strength matrix
        end
        if ~exist('alpha','var')                                                                 % If weight not input
            alpha = 0.5;                                                                         % Default to equal weighting
        end
        G             = (alpha.*combmat)+((1-alpha).*W);                                         % Create weighted matrix
        [v,d]         = eig(G,'vector');                                                         % Compute eigenvectors and values
        d(abs(d)<tol) = nan;                                                                     % Set 0 eigenvalues to nan
        [~,ind]       = max(d);                                                                  % Find maximum eigenvalue
        specc(:,1)    = abs(v(:,ind));                                                           % Centrality is associated eigenvector
end
