function [cs,cstot] = subgraph_centrality_wu(W)

%   Subgraph Centrality
%
%   [cs,cstot] = subgraph_centrality_wu(W);
%
%   Subgraph centrality of a weighted network using Crofts & Higham's
%   adjustment. Subgraph centrality assays the extent to which a node
%   participates in all subgraphs of a network, overweighting smaller
%   subgraphs. Another way to conceptualize subgraph centrality is the
%   amount (or quality) of information emanating from a node that returns
%   back to the node. The more influential a node is in all subgraphs,
%   the more it's information will dominate network communication, and thus
%   return to that node. 
%
%   Inputs:     W,        weighted, undirected connection matrix
%
%   Output:     cs,       subgraph centrality for each node
%               Ctot,     total subgraph centrality across the network,
%                         also known as the Estrada index
%               
%   References: Estrada E, Hatano N. (2008). Phys Rev E 77, 036111.
%               Crofts JJ, Higham DJ. (2009). J R Soc Interface 6, 411-414.
%
%
%   Jeff Spielberg, University of Delaware
%   
%   Copyleft 2016-Forever. Software can be modified and redistributed, but
%   modified, redistributed versions must have the same rights

%   Modification History:
%   October 2016: Original

n     = size(W,1);                           % Number of nodes
W     = W.*(W>0).*(1-eye(n));                % Ensure that all weights are positive and the diagonal is cleared
S     = diag(sum(W));                        % Compute the diagonal strength matrix
cs    = diag(expm((S^(-0.5))*W*(S^(-0.5)))); % Compute subgraph centrality
cstot = sum(cs);                             % Compute total subgraph centrality